import json
from CTADIRAC.Interfaces.API.CTAJob import CTAJob


class CtapipeProcessJob(CTAJob):
    """Job extension class for ctapipe stage1 modeling processing"""

    def __init__(self) -> None:
        super().__init__(we_type="ctapipeprocessing")
        self.setName("ctapipe_process")
        self.setType("DL0_Reprocessing")
        self.data_level = 2

    def register_log(self, meta_data_json) -> None:
        log_file_pattern = "./Data/*.log_and_prov.tgz"
        file_meta_data = {}
        file_meta_data_json = json.dumps(file_meta_data)
        output_data_type = self.output_log_type
        step = self.setExecutable(
            "cta-prod-managedata",
            arguments=f"'{meta_data_json}' '{file_meta_data_json}' {self.base_path} "
            f"'{log_file_pattern}' {self.package} {self.program_category} '{self.catalogs}' {output_data_type}",
            logFile="LogManagement_Log.txt",
            modulesList=["cta_script"],
        )
        step["Value"]["name"] = "Step_LogManagement"
        step["Value"]["descr_short"] = "Save log to SE and register them in DFC"

    def run_ctapipe_process(self) -> None:
        step = self.setExecutable(
            "./dirac_ctapipe-process_wrapper",
            arguments=f"--out_ext {self.output_extension} {self.options}",
            logFile="ctapipe_process_Log.txt",
            modulesList=["cta_script"],
        )
        step["Value"]["name"] = "Step_ctapipe_process"
        step["Value"]["descr_short"] = "Run ctapipe process"

    def run_dedicated_software(self) -> None:
        self.run_ctapipe_process()



#!/usr/bin/env python

__RCSID__ = "$Id$"

# DIRAC imports
import DIRAC
from DIRAC import gLogger
from DIRAC.Core.Base.Script import Script
from DIRAC.TransformationSystem.Client.TransformationClient import TransformationClient

Script.setUsageMessage(
    """
Treat problematic tasks for files with No replicas
Unregister files from the catalog and set transformation files to Processed

Usage:
   cta-prod-treat-problematic-transfers <ascii file with list of (transID lfn)>

Arguments:
  <ascii file with list of (transID lfn)> : input file with (transID lfn) to treat
"""
)

Script.parseCommandLine(ignoreErrors=True)
# Must comes after parseCommandLine
from DIRAC.Resources.Catalog.FileCatalog import FileCatalog

fc = FileCatalog()


def read_inputs(file_path):
    content = open(file_path).readlines()
    resList = []
    for line in content:
        transID = line.strip().split(" ")[0]
        lfn = line.strip().split(" ")[1]
        if line != "\n":
            resList.append((transID, lfn))
    return resList


def unregisterFile(lfn):
    res = fc.removeFile(lfn)
    if res["OK"]:
        if "Failed" in res["Value"]:
            if lfn in res["Value"]["Failed"]:
                gLogger.error(res["Value"]["Failed"][lfn])
            elif lfn in res["Value"]["Successful"]:
                gLogger.notice("Successfully removed from the catalog", lfn)
            else:
                gLogger.error("Unexpected error result", res["Value"])
        else:
            gLogger.notice("Successfully removed from the catalog", lfn)
    else:
        gLogger.error("Failed to remove file from the catalog:", res["Message"])

    if not res["OK"]:
        gLogger.error(res["Message"])

    return DIRAC.S_OK()


@Script()
def main():
    args = Script.getPositionalArgs()
    if len(args) != 1:
        Script.showHelp()

    infile = args[0]
    resList = read_inputs(infile)

    # Initialize the transDict with the transID keys
    transDict = {}
    for transID, lfn in resList:
        transDict.update({transID: []})

    # Unregister file from the catalog and update the transDict with values
    for transID, lfn in resList:
        res = unregisterFile(lfn)
        if not res["OK"]:
            gLogger.error(res["Message"])
            DIRAC.exit(-1)
        transDict[transID].append(lfn)

    tc = TransformationClient()

    for transID in transDict:
        res = tc.setFileStatusForTransformation(
            transID, newLFNsStatus="Processed", lfns=transDict[transID]
        )
        if not res["OK"]:
            gLogger.error(res["Message"])
            DIRAC.exit(-1)


if __name__ == "__main__":
    main()



#!/usr/bin/env python
"""
Show the progress of Moving transformations associated to one or more datasets

Warning: Moving transformations are expected to follow the naming convention:
<Move>_<MCCampaign>_<datasetName>
Transformations not following this convention can be inspected directly using the Transformation Web Monitor or the transformation CLI

Usage:
   cta-transformation-move-dataset-status <datasetName (may contain wild cards) or ascii file with a list of datasets> <MCCampaign> <target SE>

Example:
   cta-transformation-move-dataset-status Paranal_gamma_North_20deg_HB9 PROD3 CC-IN2P3-Tape
"""

__RCSID__ = "$Id$"

import os

import DIRAC
from DIRAC import gLogger
from DIRAC.Core.Base.Script import Script
from DIRAC.Resources.Catalog.FileCatalogClient import FileCatalogClient
from CTADIRAC.Core.Utilities.tool_box import read_inputs_from_file
from DIRAC.TransformationSystem.Client.TransformationClient import TransformationClient


def get_list_of_datasets(tag=""):
    fc = FileCatalogClient()
    dataset_tag = f"*{tag}*"
    res = fc.getDatasets(dataset_tag)
    if not res["OK"]:
        gLogger.error(res["Message"])
        DIRAC.exit(-1)
    dataset_dict = res["Value"]
    dataset_list = list()
    for dataset_name in dataset_dict["Successful"][dataset_tag]:
        dataset_list.append(dataset_name)
    return dataset_list


def get_dataset_info(dataset_name):
    fc = FileCatalogClient()
    res = fc.getDatasets(dataset_name)
    if not res["OK"]:
        gLogger.error("Failed to get datasets")
        DIRAC.exit(-1)
    dataset_dict = res["Value"]
    res = dataset_dict["Successful"][dataset_name][dataset_name]
    number_of_files = res["NumberOfFiles"]
    meta_query = res["MetaQuery"]
    total_size = res["TotalSize"]
    return (dataset_name, number_of_files, total_size, meta_query)


def get_transformation_info(dataset_name, MCCampaign, targetSE):
    transClient = TransformationClient()

    paramShowNames = [
        "TransformationID",
        "TransformationName",
        "Type",
        "Status",
        "Files_Total",
        "Files_PercentProcessed",
        "Files_Processed",
        "Files_Unused",
        "Jobs_TotalCreated",
        "Jobs_Waiting",
        "Jobs_Running",
        "Jobs_Done",
        "Jobs_Failed",
        "Jobs_Stalled",
    ]

    trans_name = f"Move_{MCCampaign}_{targetSE}_{dataset_name}"
    res = transClient.getTransformationParameters(trans_name, "TransformationID")

    if not res["OK"]:
        gLogger.error(f"Warning: failed to get transformation {trans_name}")
        return "_"

    transID = res["Value"]
    res = transClient.getTransformationSummaryWeb(
        {"TransformationID": transID}, [], 0, 1
    )

    if not res["OK"]:
        DIRAC.gLogger.error(res["Message"])
        DIRAC.exit(-1)

    if res["Value"]["TotalRecords"] > 0:
        paramNames = res["Value"]["ParameterNames"]
        for paramValues in res["Value"]["Records"]:
            paramShowValues = map(
                lambda pname: paramValues[paramNames.index(pname)], paramShowNames
            )
            showDict = dict(zip(paramShowNames, paramShowValues))
            files_PercentProcessed = showDict["Files_PercentProcessed"]

    return files_PercentProcessed


#########################################################
@Script()
def main():
    _, argss = Script.parseCommandLine(ignoreErrors=True)

    if len(argss) != 3:
        Script.showHelp()

    if os.path.isfile(argss[0]):
        gLogger.notice(f"\nReading datasets from input file: {argss[0]}\n")
        dataset_list = read_inputs_from_file(argss[0])
    else:
        dataset_name = argss[0]
        dataset_list = list()
        if dataset_name.find("*") > 0:
            dataset_list = get_list_of_datasets(dataset_name)
            if len(dataset_list) == 0:
                gLogger.notice("No dataset found")
                DIRAC.exit()
        else:
            dataset_list.append(dataset_name)

    MCCampaign = argss[1]
    targetSE = argss[2]

    values = []
    for dataset_name in dataset_list:
        name, n_files, size, mq = get_dataset_info(dataset_name)
        # # convert total size in TB
        size_TB = size / 1e12
        # get transformation info
        files_PercentProcessed = get_transformation_info(
            dataset_name, MCCampaign, targetSE
        )
        values.append((name, n_files, float(f"{size_TB:.2f}"), files_PercentProcessed))

    # print table
    gLogger.notice("\n|_. Name |_. N files |_. Size(TB) |_. Files Processed (%)| ")
    for value in values:
        gLogger.notice(f"|{'|'.join(map(str, value))}|")

    DIRAC.exit()


if __name__ == "__main__":
    main()



#!/usr/bin/env python
"""
Create a transformation to remove the files of a dataset (all replicas)

Usage:
   cta-transformation-remove-dataset <dataset_name>
"""

__RCSID__ = "$Id$"

import DIRAC
from DIRAC import gLogger
from DIRAC.Core.Base.Script import Script
from DIRAC.TransformationSystem.Client.Transformation import Transformation
from DIRAC.TransformationSystem.Client.TransformationClient import TransformationClient
from DIRAC.Resources.Catalog.FileCatalogClient import FileCatalogClient


def get_dataset_files(dataset_name):
    fc = FileCatalogClient()
    res = fc.getDatasetFiles(dataset_name)
    if not res["OK"]:
        gLogger.error("Failed to get files for dataset:", res["Message"])
        DIRAC.exit(-1)
    else:
        lfn_list = res["Value"]["Successful"][dataset_name]
        gLogger.notice(f"Found {len(lfn_list)} files for removal")
    return lfn_list


@Script()
def main():
    Script.parseCommandLine(ignoreErrors=True)
    argss = Script.getPositionalArgs()
    if len(argss) != 2:
        Script.showHelp()

    dataset_name = argss[0]
    lfn_list = get_dataset_files(dataset_name)

    group_size = int(argss[1])

    t = Transformation()
    tc = TransformationClient()
    trans_name = f"Removing_{dataset_name}"
    t.setTransformationName(trans_name)
    t.setType("Removal")
    t.setDescription("Removing file")
    t.setLongDescription("Removing file")
    t.setGroupSize(group_size)
    t.setPlugin("Standard")
    transBody = "Removal;RemoveFile"
    t.setBody(transBody)

    res = t.addTransformation()
    if not res["OK"]:
        gLogger.error(res["Message"])
        DIRAC.exit(-1)

    transID = t.getTransformationID()
    res = tc.addFilesToTransformation(transID["Value"], lfn_list)
    if not res["OK"]:
        gLogger.error(res["Message"])
        DIRAC.exit(-1)

    t.setAgentType("Automatic")
    t.setStatus("Active")

    DIRAC.exit()


#########################################################
if __name__ == "__main__":
    main()



from typing import Union
from DIRAC.TransformationSystem.Client.TransformationClient import TransformationStatus
from DIRAC.WorkloadManagementSystem.Client import JobStatus
from DIRAC.ConfigurationSystem.Client.Helpers.Resources import getSites
import typer


def transformation_type_callback(value: str) -> Union[list[str], None]:
    trans_types: list[str] = value.split(",")
    allowed_types: list[str] = ["MCSimulation", "Processing", "Merging", "All"]
    for t_type in trans_types:
        if t_type not in allowed_types:
            raise typer.BadParameter(
                f"Bad transformation type '{t_type}'. Allowed types: {allowed_types}"
            )
        if t_type == "All":
            return None
    return trans_types


def transformation_status_callback(value: str) -> Union[list[str], None]:
    allowed_status: list[str] = TransformationStatus.TRANSFORMATION_STATES + ["All"]
    status_list: list[str] = value.split(",")
    for status in status_list:
        if status not in allowed_status:
            raise typer.BadParameter(
                f"Bad transformation status '{status}'. Allowed status: {allowed_status}"
            )
        if status == "All":
            return None
    return status_list


ALLOWED_PARAMETERS: list[str] = [
    "AgentLocalSE",
    "CEQueue",
    "CPUNormalizationFactor",
    "DiskSpace",
    "GridCE",
    "HostName",
    "JobType",
    "JobWrapperPID",
    "LastUpdateCPU",
    "LoadAverage",
    "LocalAccount",
    "MatcherServiceTime",
    "Memory",
    "MemoryUsed",
    "ModelName",
    "NormCPUTime",
    "OutputSandboxMissingFiles",
    "PayloadPID",
    "PilotAgent",
    "Pilot_Reference",
    "ScaledCPUTime",
    "TotalCPUTime",
    "WallClockTime",
]

TRUE_PARAMETERS: list[str] = [
    "AgentLocalSE",
    "CEQueue",
    "CPUNormalizationFactor",
    "DiskSpace(MB)",
    "GridCE",
    "HostName",
    "JobType",
    "JobWrapperPID",
    "LastUpdateCPU(s)",
    "LoadAverage",
    "LocalAccount",
    "MatcherServiceTime",
    "Memory(MB)",
    "MemoryUsed(MB)",
    "ModelName",
    "NormCPUTime(s)",
    "OutputSandboxMissingFiles",
    "PayloadPID",
    "PilotAgent",
    "Pilot_Reference",
    "ScaledCPUTime(s)",
    "TotalCPUTime(s)",
    "WallClockTime(s)",
]
PARAMETER_DICT = dict(zip(ALLOWED_PARAMETERS, TRUE_PARAMETERS))


def transformation_or_job_id_callback(value: str):
    value = value.split(",")
    try:
        for val in value:
            int(val)
    except (TypeError, AttributeError, ValueError):
        raise typer.BadParameter(f"Bad parameters type '{val}' should be an integer")
    return value


def job_parameters_callback(value: str) -> str:
    if value not in ALLOWED_PARAMETERS:
        raise typer.BadParameter(f"Bad parameters '{value}'")
    return PARAMETER_DICT[value]


def job_site_callback(value: str):
    res_sites = getSites()
    sites = []
    if res_sites["OK"]:
        sites = res_sites["Value"]
    values = value.split(",")
    for val in values:
        if val not in sites:
            raise typer.BadParameter(f"Bad sites '{value}'. Allowed sites: {sites}")
    return values


def job_status_callback(value: str) -> Union[list[str], None]:
    allowed_status: list[str] = JobStatus.JOB_FINAL_STATES + ["All"]
    status_list: list[str] = value.split(",")
    for status in status_list:
        if status not in allowed_status:
            raise typer.BadParameter(
                f"Bad job status '{status}'. Allowed status: {allowed_status}"
            )
        if status == "All":
            return None
    return status_list


ALLOWED_JOB_FIELDS = {
    "JobGroup": transformation_or_job_id_callback,
    "Site": job_site_callback,
    "Status": job_status_callback,
}

ALLOWED_TRANSFROMATION_FIELDS = {
    "TransformationID": transformation_or_job_id_callback,
    "Status": transformation_status_callback,
    "Type": transformation_type_callback,
}


def transformation_fields_arg_callback(values: list) -> list[dict]:
    argument_dict = {}
    for i in range(0, len(values), 2):
        arg = values[i]
        value = values[i + 1]
        if arg in ALLOWED_TRANSFROMATION_FIELDS:
            trans_value = ALLOWED_TRANSFROMATION_FIELDS[arg](value)
            argument_dict[arg] = trans_value
        else:
            raise typer.BadParameter(
                f"Argument '{arg}' not in allowed transformation fields: {list(ALLOWED_TRANSFROMATION_FIELDS.keys())}"
            )
    return [argument_dict]


def job_fields_arg_callback(values: list) -> list[dict]:
    argument_dict = {}
    for i in range(0, len(values), 2):
        arg = values[i]
        value = values[i + 1]
        if arg in ALLOWED_JOB_FIELDS:
            trans_value = ALLOWED_JOB_FIELDS
            argument_dict[arg] = trans_value[arg](value)
        else:
            raise typer.BadParameter(
                f"Argument '{arg}' not in allowed job fields: {list(ALLOWED_JOB_FIELDS.keys())}"
            )
    return [argument_dict]


PRODUCTION_FIELDS: list[str] = [
    "ProductionID",
    "ProductionName",
    "Status",
    "CreationDate",
    "LastUpdate",
    "AuthorDN",
    "AuthorGroup",
]


def cond_dict_callback(value: str | None):
    if value is None:
        return None
    cond_dict = eval(value)
    if not isinstance(cond_dict, dict):
        example = "{'ProductionID': [1,2,3]}"
        raise typer.BadParameter(f"'{value}' should be a dict type: '{example}'")
    else:
        for key in cond_dict.keys():
            if key not in PRODUCTION_FIELDS:
                raise typer.BadParameter(
                    f"'{key}' is a bad key.\nAvailable keys: {PRODUCTION_FIELDS}"
                )
    return cond_dict



from typing import Literal, TypedDict
from DIRAC import S_OK


class DReportReturnType(TypedDict):
    """used for typing the DIRAC return structure"""

    OK: Literal[True]
    Value: dict[bool, str]


def s_report(report: bool = False, message: str = "") -> DReportReturnType:
    """Return a S_OK but with a report bool"""
    return S_OK({"Report": report, "Message": message})



#!/usr/bin/env python
"""
Move a dataset distributed on on number of production SEs:
'DESY-ZN-Disk', 'LPNHE-Disk', 'CNAF-Disk', 'CYF-STORM-Disk','LAPP-Disk', 'CEA-Disk', 'CC-IN2P3-Disk', 'LANCASTER-Disk', 'POLGRID-Disk'
to a given destination SE, e.g. CC-IN2P3-Tape

Usage:
cta-transformation-move-dataset <dataset name or ascii file with a list of datasets> <dest SE>

Optional arguments:
<group size>: size of the transformation (default=1)
<extra_tag>: extra tag to be appended to the name of the transformation

Example:
cta-transformation-move-dataset Prod4_Paranal_gamma_North_20deg_SSTOnly_MC0 CC-IN2P3-Tape 100

Warning:
If the <extra_tag> option is used the transformations will not be found by the cta-prod-move-status command and should be monitored using
the transformation CLI or the Transformation Web Monitor
"""

__RCSID__ = "$Id$"

import os

import DIRAC
from DIRAC import gLogger
from DIRAC.Core.Base.Script import Script
from DIRAC.Resources.Catalog.FileCatalogClient import FileCatalogClient
from DIRAC.TransformationSystem.Utilities.ReplicationTransformation import (
    createDataTransformation,
)
from CTADIRAC.Core.Utilities.tool_box import read_inputs_from_file


def get_dataset_info(dataset_name):
    """Return essential dataset information
    Name, number of files, total size and meta query
    """
    fc = FileCatalogClient()
    res = fc.getDatasets(dataset_name)
    if not res["OK"]:
        gLogger.error(res["Message"])
        DIRAC.exit(-1)
    dataset_dict = res["Value"]
    res = dataset_dict["Successful"][dataset_name][dataset_name]
    number_of_files = res["NumberOfFiles"]
    meta_query = res["MetaQuery"]
    total_size = res["TotalSize"]
    return (dataset_name, number_of_files, total_size, meta_query)


@Script()
def main():
    dataset_list = []
    Script.parseCommandLine(ignoreErrors=True)
    argss = Script.getPositionalArgs()
    if len(argss) < 2:
        Script.showHelp()
    if os.path.isfile(argss[0]):
        gLogger.notice("Reading datasets from input file:", argss[0])
        dataset_list = read_inputs_from_file(argss[0])
    else:
        dataset_list.append(argss[0])

    dest_se = argss[1]
    group_size = 1
    extra_tag = ""

    if len(argss) >= 3:
        group_size = argss[2]
    if len(argss) == 4:
        extra_tag = argss[3]

    # Check input data set information
    for dataset_name in dataset_list:
        name, n_files, size, meta_query = get_dataset_info(dataset_name)
        gLogger.notice("Found dataset %s with %d files." % (name, n_files))
        gLogger.debug(meta_query)
        # choose a metaKey
        # To do: avoid hard coded metakey
        if "MCCampaign" in meta_query:
            meta_key = "MCCampaign"
            meta_value = meta_query["MCCampaign"]
        else:
            gLogger.error("Error: MCCampaign metadata not defined")
            DIRAC.exit(-1)

        tag = dataset_name
        if extra_tag:
            tag = tag + "_" + extra_tag

        do_it = True
        # To do: replace hard coded SE with SE read from CS
        source_se = [
            "DESY-ZN-Disk",
            "LPNHE-Disk",
            "CNAF-Disk",
            "CYF-STORM-Disk",
            "LAPP-Disk",
            "CEA-Disk",
            "CC-IN2P3-Disk",
            "POLGRID-Disk",
            "LANCASTER-Disk",
        ]

        # create Transformation
        res = createDataTransformation(
            flavour="Moving",
            targetSE=dest_se,
            sourceSE=source_se,
            metaKey=meta_key,
            metaValue=meta_value,
            extraData=meta_query,
            extraname=tag,
            groupSize=int(group_size),
            plugin="Broadcast",
            tGroup=None,
            tBody=None,
            enable=do_it,
        )
        if not res["OK"]:
            gLogger.error(res["Message"])
            DIRAC.exit(-1)

    DIRAC.exit()


#########################################################
if __name__ == "__main__":
    main()



from DIRAC.TransformationSystem.Service.TransformationManagerHandler import (
    TransformationManagerHandler as TManagerBase,
)


class TransformationManagerHandlerMixin:
    types_get_tasks_by_file_status = [int, str]

    @classmethod
    def export_get_tasks_by_file_status(cls, trans_id, file_status, task_status=None):
        return cls.transformationDB.get_tasks_by_file_status(
            trans_id, file_status, task_status
        )


class TransformationManagerHandler(TransformationManagerHandlerMixin, TManagerBase):
    pass
